/*
**  Copyright (C) 1993/1994, OFFIS, Oldenburg University and CERIUM
**  
**  This software and supporting documentation were
**  developed by 
**  
**    Institut OFFIS
**    Bereich Kommunikationssysteme
**    Westerstr. 10-12
**    26121 Oldenburg, Germany
**    
**    Fachbereich Informatik
**    Abteilung Prozessinformatik
**    Carl von Ossietzky Universitaet Oldenburg 
**    Ammerlaender Heerstr. 114-118
**    26111 Oldenburg, Germany
**    
**    CERIUM
**    Laboratoire SIM
**    Faculte de Medecine
**    2 Avenue du Pr. Leon Bernard
**    35043 Rennes Cedex, France
**  
**  for CEN/TC251/WG4 as a contribution to the Radiological 
**  Society of North America (RSNA) 1993 Digital Imaging and 
**  Communications in Medicine (DICOM) Demonstration.
**  
**  THIS SOFTWARE IS MADE AVAILABLE, AS IS, AND NEITHER OFFIS,
**  OLDENBURG UNIVERSITY NOR CERIUM MAKE ANY WARRANTY REGARDING 
**  THE SOFTWARE, ITS PERFORMANCE, ITS MERCHANTABILITY OR 
**  FITNESS FOR ANY PARTICULAR USE, FREEDOM FROM ANY COMPUTER 
**  DISEASES OR ITS CONFORMITY TO ANY SPECIFICATION.  THE 
**  ENTIRE RISK AS TO QUALITY AND PERFORMANCE OF THE SOFTWARE   
**  IS WITH THE USER. 
**  
**  Copyright of the software and supporting documentation
**  is, unless otherwise stated, jointly owned by OFFIS,
**  Oldenburg University and CERIUM and free access is hereby
**  granted as a license to use this software, copy this
**  software and prepare derivative works based upon this
**  software. However, any distribution of this software
**  source code or supporting documentation or derivative
**  works (source code and supporting documentation) must
**  include the three paragraphs of this copyright notice. 
** 
*/
/*
** CTN Common Constants, Types, Globals and Functions
**
** Author: 	Andrew Hewett
**		Universitaet Oldenburg, Germany
** Created:	08/1993
**
** Last Update:		hewett
** Update Date:		1996/04/22 10:20:11
** Source File:		/share/dicom/cvs-depot/dcmtk/imagectn/apps/imagectn.h,v
** CVS/RCS Revision:	1.2
** Status:		Exp
**
** CVS/RCS Log at end of file
*/


#ifndef CTN_H
#define CTN_H


#include "osconfig.h"    /* make sure OS specific configuration is included first */

#include "dicom.h"
#include "cond.h"
#include "assoc.h"
#include "cnf.h"

/*
 * Application Conditions
 */

#define APP_NORMAL		/* normal operation */\
	FORM_COND(FAC_APP, SEV_SUCC, 0)
#define APP_BADIMAGE		/* bad image data */\
	FORM_COND(FAC_APP, SEV_ERROR, 1)
#define APP_NOCONTEXT		/* no presentation context for SOP Class */\
	FORM_COND(FAC_APP, SEV_ERROR, 2)
#define APP_PROTOCOLERROR	/* protocol error */\
	FORM_COND(FAC_APP, SEV_ERROR, 3)
#define APP_MESSAGEFAILED	/* network message failed */\
	FORM_COND(FAC_APP, SEV_ERROR, 4)
#define APP_STOREFAILED		/* store operation failed */\
	FORM_COND(FAC_APP, SEV_ERROR, 5)
#define APP_INVALIDPEER		/* invalid peer (for move operation) */\
	FORM_COND(FAC_APP, SEV_ERROR, 6)

/*
 * Common Globals (defined in imagectn.c)
 */

extern T_ASC_Network *net;

extern char* progname;
extern BOOLEAN verbose;
extern BOOLEAN debug;
extern BOOLEAN ignoreStoreData;
extern BOOLEAN requireFindForMove;
extern BOOLEAN displayImagesOnMove;
extern BOOLEAN displayTransferProgress;
extern BOOLEAN restrictMoveToSameVendor;
extern BOOLEAN restrictMoveToSameHost;
extern BOOLEAN restrictMoveToSameAE;

/* tables of abstract syntaxes which image ctn supports */
extern char* nonStorageSyntaxes[];
extern int numberOfNonStorageSyntaxes;
extern char* storageSyntaxes[];
extern int numberOfStorageSyntaxes;

/*
 * Common Function Definitions
 */

extern void errmsg(const char* msg, ...);

extern BOOLEAN peerCanDisplayImages(const char* peerName);

#endif

/*
** CVS Log
** imagectn.h,v
** Revision 1.2  1996/04/22 10:20:11  hewett
** Added global variables for restricting move destinations.
**
** Revision 1.1.1.1  1996/03/28 19:24:59  hewett
** Oldenburg Image CTN Software ported to use the dcmdata C++ toolkit.
**
**
*/
