/*
** Modality Worklist Database
**
** Author: 	Sandra von Gehlen
**		Oldenburg University, Germany
** Created:	03/1996
**
** Last Update:		hewett
** Update Date:		1996/05/30 17:55:22
** Source File:		/share/dicom/cvs-depot/dcmtk/wlistctn/libsrc/wrklstdb.cc,v
** CVS/RCS Revision:	1.8
** Status:		Exp
**
** CVS/RCS Log at end of file
*/

#include "osconfig.h"

#include <stdio.h>
#include <string.h>
#include <stream.h>
#ifdef HAVE_SYS_TYPES_H
#include <sys/types.h>
#endif
#ifdef HAVE_SYS_STAT_H
#include <sys/stat.h>
#endif
#ifdef HAVE_FCNTL_H
#include <fcntl.h>
#endif
#ifdef HAVE_UNISTD_H
#include <unistd.h>
#endif
#ifdef HAVE_STDLIB_H
#include <stdlib.h>
#endif
#ifdef HAVE_LIBC_H
#include <libc.h>
#endif
#include "dctk.h"
#include "dcdebug.h"
#include "dcvm.h"
#include "dctypes.h"
#include "dcstack.h"
#include "dclist.h"
#include "dcvrat.h"
#include "wrklstdb.h"

#define LOCKFILENAME "lockfile"

char * standardizeDate(const char * da)
{
    if (!da)
	return NULL;
    
    if (strlen(da) < 8)
	return NULL;
    
    char * stdDate = new char[9];
    stdDate[8] = 0;
    int i = 0;
    int error = 0;
    const char * ptr = da;
    
    while (*ptr && ! error)
    {
	if (*ptr >= '0' && *ptr <='9')
	{
	    if (i < 8)
		stdDate[i] = *ptr;
	    i++;
	}
	else if ((*ptr != '.' || (i != 4 && i != 6)) && *ptr != ' ')      
	    error = 1;
	
	ptr++;
    }
    
    if (error || i != 8)
    { 
	delete[] stdDate;
	return NULL;
    }
    else
	return stdDate;
}

char * standardizeTime(const char * tm)
{
    if (!tm)
	return NULL;
    
    if (strlen(tm) < 2)
	return NULL;
    
    char * stdTime = new char[14];
    stdTime[13]=0;
    int i = 0;
    int error = 0;
    const char * ptr = tm;
    
    while(*ptr && !error)
    {
	if ((*ptr >= '0' && *ptr <='9') ||
	    (*ptr == '.' && i == 6))
	{
	    if (i < 13)
		stdTime[i] = *ptr;
	    i++;
	}
	else if ((*ptr != ':' || (i != 2 && i != 4)) && *ptr != ' ')      
	    error = 1;
	
	
	ptr++;
    }
    
    if ( i < 2 || ((i & 1) && i<6 ))
	error = 1;
    
    if (!error)
    {
	if (i < 6)
	    for (;i < 6; i++)
		stdTime[i] = '0';
	
	if (i==6)
	    stdTime[i++] = '.';
	
	if (i < 13)
	    for(; i < 13; i++)
		stdTime[i] = '0';
	
    }
    
    if (error)
    {
	delete[] stdTime;
	return NULL;
    }
    else
	return stdTime;
}

int WorklistDB::set_readlock(const char *path)
{
    struct flock lockdata;
  char lockname[1024];
  int result;

  if (path == NULL)
  {
    fprintf(stderr,"error: NULL path not allowed\n");
    return 0;
  }

  if (fd != 0)
  {
    fprintf(stderr,"error: nested read locks not allowed!\n");
    return 0;
  }

  strcpy(lockname,path);
  if ((strlen(lockname)>0)&&(lockname[strlen(lockname)-1] != '/'))
    strcat(lockname,"/");
  strcat(lockname,LOCKFILENAME);

  fd=open(lockname, O_RDWR);
  if (fd == -1)
  {
    fd=0;
    fprintf(stderr,"error: cannot open file %s\n",lockname);
    perror("return code");
    return 0;
  }

  lockdata.l_type = F_RDLCK;
  lockdata.l_whence=0;
  lockdata.l_start=0;
  lockdata.l_len=0;
  result = fcntl(fd, F_SETLKW, &lockdata);
  if (result == -1)
  {
    fprintf(stderr, "error: cannot set read lock on file %s\n",lockname);
    perror("return code");
    close(fd);
    fd=0;
    return 0;
  }

  return 1;
}

int WorklistDB::release_readlock()
{
  struct flock lockdata;
  int result;

  if (fd==0)
  {
    fprintf(stderr,"error: no readlock to release\n");
    return 0;
  }

  lockdata.l_type = F_UNLCK;
  lockdata.l_whence=0;
  lockdata.l_start=0;
  lockdata.l_len=0;
  result = fcntl(fd, F_SETLKW, &lockdata);
  if (result == -1)
  {
    fprintf(stderr, "error: cannot release read lock\n");
    perror("return code");
    close(fd);
    fd=0;
    return 0;
  }

  close(fd);
  fd=0;
  return 1;
}


static
BOOL wcmatch(const char * string, const char * wc);

static
BOOL matchstar(const char * string, const char * wc)
{
    BOOL ok = FALSE;
    const char * strptr = string;
    const char * wcptr = wc;

    while (*wcptr == '*')
	wcptr++;

    if (*wcptr == 0)
	return TRUE;

    while (!ok && *strptr != 0)
    {
	if (*wcptr == '?' || *strptr == *wcptr)
	    ok = wcmatch(strptr+1, wcptr+1);
	if (!ok)
	    strptr++;
    }
    return ok;
}
	   
static
BOOL wcmatch(const char * string, const char * wc)
{
    if (wc == NULL)
	return FALSE;

    const char * strptr;
    if (string == NULL)
	strptr = "";
    else
	strptr = string;
    const char * wcptr = wc;
    BOOL ok = FALSE;

    while(*wcptr != 0 && *wcptr != '*' && 
	  ((*wcptr == '?' && *strptr != 0) ||  *wcptr == *strptr))
    {
	wcptr++;
	strptr++;
    }
	
    if (*wcptr == '*')
	ok = matchstar(strptr, wcptr+1);
    else 
	ok = *wcptr == 0 && *strptr == 0;

    return ok;
}

static BOOL matchPattern(const char* value, const char* pattern)
{
    /*
    ** Matches the pattern against a specific value.
    */
    BOOL ok = FALSE;

    if (pattern == NULL || pattern[0] == '\0') {
	/* universal matching */
	ok = TRUE;
    } else if (wcmatch(value, pattern)) {
	/* single value matching & wildcard matching (wcmatch does both) */
	ok = TRUE;
    }
    return ok;
}

static BOOL
rangematchTime (const char * s1, const char * s2)
{
    if (s1 == NULL || s2 == NULL)
	return FALSE;
    char * t;
    BOOL ok = FALSE;

    int len = strlen(s2);
    t = new char[len+1];
    strcpy(t, s2);
    char * strich = strchr(t, '-');
    
    if (strchr(t, '-'))
    {
 	if (t[0] == '-')
 	{
	    char *n1;
	    n1 = standardizeTime(&t[1]);
	    if (n1 != NULL)
	    {
		if (strcmp(s1, n1) <= 0)
		    ok = TRUE;
		else 
		    ok = FALSE;
	    }
	    else
		ok = FALSE;
	    delete n1;
	}
 	else if ((t[len-1] == '-') && (t[len] == '\0'))
 	{
	    char *n1;
 	    t[len-1] ='\0';
	    n1 =  standardizeTime(t);
	    if (n1 != NULL)
	    {
		if (strcmp(s1, n1) >= 0)
		    ok = TRUE;
		else
		    ok = FALSE;
	    }
	    else
		ok = FALSE;
	    delete n1;
	}
	else
 	{
	   * strich = 0;
	   strich ++;
	   char *n2;
	   char *n1;
	   n1 = standardizeTime(t);
	   n2 = standardizeTime(strich);
	   if ((n1 != NULL)&&(n1 != NULL))
	   {
	       if ((strcmp(s1, n1) >= 0) && (strcmp(s1, n2) <= 0)) 
		   ok = TRUE;
	       else
		   ok = FALSE;
	   }
	   else 
	       ok = FALSE;
	   delete n1;
	   delete n2;
	}
    }
    else
 	ok = FALSE;
    delete t;
    return ok;
}

static BOOL
matchTime(const char* value, const char* pattern)
{
    BOOL ok = FALSE;;

    if (pattern == NULL || pattern[0] == '\0') {
	/* universal matching */
	ok = TRUE;
    } else if (value == NULL || value[0] == '\0') {
	/* pattern has contents but value is empty */
	ok = FALSE;
    } else {
	char *stdValue;
	stdValue = standardizeTime(value);
	if (!strchr(pattern,'-'))
	{
	    char *stdPattern;
	    stdPattern = standardizeTime(pattern);

	    ok = (strcmp(stdValue, stdPattern) == 0);

	    delete stdPattern;
	} else {
	    ok = rangematchTime(stdValue, pattern);
	}
	delete stdValue;
    }
    return ok;
}

static BOOL
rangematchDate (const char * s1, const char * s2)
{
    if (s1 == NULL || s2 == NULL)
	return FALSE;
    char * t;
    BOOL ok = FALSE;

    int len = strlen(s2);
    t = new char[len+1];
    strcpy(t, s2);
    char * strich = strchr(t, '-');
    
    if (strchr(t, '-'))
    {
 	if (t[0] == '-')
 	{
	    char *n1;
	    n1 = standardizeDate(&t[1]);
	    if (n1 != NULL)
	    {
		if (strcmp(s1, n1) <= 0)
		    ok = TRUE;
		else 
		    ok = FALSE;
	    }
	    else
		ok = FALSE;
	    delete n1;
	}
 	else if ((t[len-1] == '-') && (t[len] == '\0'))
 	{
	    char *n1;
 	    t[len-1] ='\0';
	    n1 =  standardizeDate(t);
	    if (n1 != NULL)
	    {
		if (strcmp(s1, n1) >= 0)
		    ok = TRUE;
		else
		    ok = FALSE;
	    }
	    else
		ok = FALSE;
	    delete n1;
	}
	else
 	{
	   * strich = 0;
	   strich ++;
	   char *n2;
	   char *n1;
	   n1 = standardizeDate(t);
	   n2 = standardizeDate(strich);
	   if ((n1 != NULL)&&(n2 != NULL))
	   {
	       if ((strcmp(s1, n1) >= 0) && (strcmp(s1, n2) <= 0)) 
		   ok = TRUE;
	       else
		   ok = FALSE;
	   }
	   else
	       ok = FALSE;
	   delete n1;
	   delete n2;
	}
    }
    else
 	ok = FALSE;
    delete t;
    return ok;
}

static BOOL
matchDate(const char* value, const char* pattern)
{
    BOOL ok = FALSE;;

    if (pattern == NULL || pattern[0] == '\0') {
	/* universal matching */
	ok = TRUE;
    } else if (value == NULL || value[0] == '\0') {
	/* pattern has contents but value is empty */
	ok = FALSE;
    } else {
	char *stdValue;
	stdValue = standardizeDate(value);
	if (!strchr(pattern,'-'))
	{
	    char *stdPattern;
	    stdPattern = standardizeDate(pattern);

	    ok = (strcmp(stdValue, stdPattern) == 0);

	    delete stdPattern;
	} else {
	    ok = rangematchDate(stdValue, pattern);
	}
	delete stdValue;
    }
    return ok;
}

static BOOL
isaSupportedMatchingKey(const DcmTagKey& key, int level) 
{
    BOOL ok = FALSE;

    if (level == 0) {
	ok = (key == DCM_PatientName) || 
	    (key == DCM_PatientID) ||
	    (key == DCM_ScheduledProcedureStepSequence);
    } else if (level == 1) {
	ok = (key == DCM_ScheduledStationAETitle) || 
	    (key == DCM_ScheduledProcedureStepStartDate) ||
	    (key == DCM_ScheduledProcedureStepStartTime) || 
	    (key == DCM_Modality) ||
	    (key == DCM_ScheduledPerformingPhysiciansName) ||
	    (key == DCM_ScheduledStationName) ||
	    (key == DCM_ScheduledProcedureStepLocation) ||
	    (key == DCM_CommentsOnTheScheduledProcedureStep);
    }
    return ok;
}

static BOOL
isaSupportedReturnKey(const DcmTagKey& key, int level) 
{
    BOOL ok = FALSE;

    if (level == 0) {
	ok = isaSupportedMatchingKey(key, level) ||
	    (key == DCM_SpecificCharacterSet) || 
	    (key == DCM_MedicalAlerts) ||
	    (key == DCM_ContrastAllergies) || 
	    (key == DCM_RequestedProcedureID) ||
	    (key == DCM_RequestedProcedureDescription) || 
	    (key == DCM_StudyInstanceUID) ||
	    (key == DCM_RequestedProcedurePriority) || 
	    (key == DCM_AccessionNumber) ||
	    (key == DCM_RequestingPhysician) || 
	    (key == DCM_PatientBirthDate) || 
	    (key == DCM_PatientSex);
    } else if (level == 1) {
	ok = isaSupportedMatchingKey(key, level) ||
	    (key == DCM_ScheduledProcedureStepDescription) ||
	    (key == DCM_PreMedication) ||
	    (key == DCM_ScheduledProcedureStepID) ||
	    (key == DCM_RequestedContrastAgent);
    }
    return ok;
}

static char *
getvalue(DcmElement* elem)
{
    char *val = NULL;
    DcmEVR v = elem->ident();
    if ((v == EVR_SH) || (v == EVR_PN) || (v == EVR_LO) || (v == EVR_LT))
    {
	val = ((DcmCharString*)elem)->get();
    }
    else if ((v == EVR_CS) || (v == EVR_DA) || (v == EVR_UI) || (v == EVR_AE) || (v == EVR_TM) )
    {
	val =((DcmByteString*)elem)->get();
    }
    return val;
}

static char *
getvalue(DcmItem* dset, const DcmTag &tag)
{
    DcmStack stk;
    char *val;
    dset->search(tag, stk, ESM_fromHere, FALSE);
    DcmObject *obj = stk.top();
    if (obj == NULL)
	val = NULL;
    else
    {
	val = getvalue((DcmElement*)obj);
    }
    return val;
}


// *******************************


WorklistDB::WorklistDB(const char* dbDirectoryPath)
{
    // make sure data dictionary is loaded 
    if (dcmDataDict.numberOfEntries() == 0) {
	fprintf(stderr, "Warning: no data dictionary loaded, check environment variabe: %s\n",
  		DCM_DICT_ENVIRONMENT_VARIABLE);
    }
    fd = 0;
    dirpath = dbDirectoryPath;
    objlist = new DcmList();
    identifiers = new DcmDataset();
    offendingElements = new DcmAttributeTag(DCM_OffendingElement, 0);
    errorElements = new DcmAttributeTag(DCM_OffendingElement, 0);
    errorComment = new DcmLongString(DCM_ErrorComment, 0);
    foundUnsupportedOptionalKey = FALSE;
}

// *******************************

WorklistDB::~WorklistDB()
{
    this -> clearObjectList();
    delete objlist;
    this -> clearDataset(identifiers);
    delete identifiers;
    delete offendingElements;
    delete errorElements;
    delete errorComment;
    if (fd != 0) release_readlock();
}

// *******************************

void WorklistDB::clearObjectList(void)
{
    if (objlist && objlist -> card())
    {
	DcmObject * removeObj = NULL;

	objlist -> seek(ELP_first);
	for(unsigned long i = 0; i < objlist->card(); i++)
	{
	    removeObj = objlist->remove();
	    if (removeObj)
		delete removeObj;
	}
    }
}

// *******************************

void WorklistDB::clearDataset(DcmDataset* identifiers)
{
    if ( identifiers && (identifiers -> card() > 0))
    {
	identifiers->clear();
    }
}

// *******************************

void WorklistDB::putOffendingElements(DcmTag& tag)
{
    DcmTag errortag;
    unsigned long d = offendingElements->getVM();
		    
    if ( d == 0)
    {
	offendingElements->put(tag, 0);
    }	
    else 
    {
	for (unsigned long j = 0; j < d; j++)
	{
	    errortag = offendingElements->get(j);
	    if (errortag != tag)
	    {
		offendingElements->put(tag, d);				                     
	    }
	}
    }
}

// *******************************

void WorklistDB::putErrorElements(DcmTag& tag)
{
    DcmTag errortag;
    unsigned long d = errorElements->getVM();
		    
    if ( d == 0)
    	errorElements->put(tag, 0);
    else 
	errorElements->put(tag, d);				                     
}

//*****************************


BOOL WorklistDB::checkIdentifiers(DcmDataset* identifiers)
{
    BOOL ok = TRUE;
    unsigned long cids = identifiers->card();

    foundUnsupportedOptionalKey = FALSE;
       
    for (unsigned long i = 0; i < cids ; i++)
    {
	DcmElement *element = identifiers->getElement(i);
	Uint32 laenge = element->getLength();
	DcmTag tag(element->getTag());
      	if (element->ident() != EVR_SQ)
	{
	    if (isaSupportedMatchingKey(tag.getXTag(), 0)) {
		/* all ok */
	    } else if (isaSupportedReturnKey(tag.getXTag(), 0)) {
		if (laenge != 0) {
		    foundUnsupportedOptionalKey = TRUE;
		}
	    } else {
		foundUnsupportedOptionalKey = TRUE;
		this->putErrorElements(tag);
	    }
	}
	else
	{
	    if (isaSupportedMatchingKey(tag.getXTag(), 0)) {
		/* check inside the sequence */
		if (laenge != 0) 
		{
		    DcmSequenceOfItems *sequits = (DcmSequenceOfItems*)element;
		    if (sequits->card() > 1) {
			/* we may not have more that 1 item in the request identifiers */
			this->putOffendingElements(tag);
			errorComment->put("more than 1 item in sequence");
			return FALSE;
		    }

		    DcmItem *item = sequits -> getItem(0);
		    unsigned long cit = item -> card();
		    for (unsigned long k = 0; k < cit; k++)
		    { 
			DcmElement *elementseq = item -> getElement(k);
			Uint32 laenge1 = elementseq -> getLength();
			DcmTag tagseq(elementseq->getTag());
			if (isaSupportedMatchingKey(tagseq.getXTag(), 1)) {
			    /* all ok */
			} else if (isaSupportedReturnKey(tagseq.getXTag(), 1)) {
			    if (laenge1 != 0) {
				foundUnsupportedOptionalKey = TRUE;
			    }
			} else {
			    foundUnsupportedOptionalKey = TRUE;
			    this->putErrorElements(tagseq);
			}
		    }

#ifdef DELETE_UNEXPECTED_ELEMENTS_FROM_IDENTIFIER
		    unsigned long d = errorElements->getVM();
		    for (unsigned long e = 0; e < d; e++)
		    {
			DcmTag errortag;
			errortag = errorElements->get(e);
			delete item -> remove(errortag);
		    }
#endif
		}
	    } else if (isaSupportedReturnKey(tag.getXTag(), 0)) {
		if (laenge != 0) {
		    foundUnsupportedOptionalKey = TRUE;
		}
	    } else {
		foundUnsupportedOptionalKey = TRUE;
		this->putErrorElements(tag);
	    }
	}
    }
#ifdef DELETE_UNEXPECTED_ELEMENTS_FROM_IDENTIFIER
    unsigned long d = errorElements->getVM();
    for (unsigned long h = 0; h < d; h++)
    {
	DcmTag errortag;
	errortag = errorElements->get(h);
	delete identifiers -> remove(errortag);
    }
#endif

    return ok;
}
		
BOOL WorklistDB::checkWklist(DcmDataset* dsetfile, const char* fname)
{
    BOOL ok = FALSE;
    DcmStack stk;
    dsetfile->search(DCM_ScheduledProcedureStepSequence, stk);
    DcmObject *obj = stk.top();
    if (obj == NULL)
    {
	ok = FALSE;
	cerr << "WLDB ERROR: missing ScheduledProcedureStepSequence in worklist database file: " 
	     << fname << endl;
    }
    else
    {
	unsigned long cdset = dsetfile->card();
	for (unsigned long i = 0; i < cdset ; i++)
	{
	    DcmElement *element = dsetfile->getElement(i);
	    Uint32 laenge = element->getLength();
	    DcmTag tag(element->getTag());

	    if (isaSupportedReturnKey(tag.getXTag(), 0)) 
		ok = TRUE;
	    else 
	    {
		ok = FALSE;
		cerr << "WLDB ERROR: unsupported attribute: " 
		     << tag << " " << tag.getTagName() << endl;
		cerr << "            in worklist database file: " << fname << endl;
		break;
	    }

	    if (element->ident() == EVR_SQ)
	    {
		if (laenge == 0)
		    ok = TRUE;
		else
		{
		    DcmSequenceOfItems *sequits = (DcmSequenceOfItems*)element;
		    DcmItem *item = sequits -> getItem(0);
		    unsigned long cit = item -> card();
		    for (unsigned long k = 0; k < cit; k++)
		    { 
			DcmElement *elementseq = item -> getElement(k);
			DcmTag tagseq(elementseq->getTag());
			if (isaSupportedReturnKey(tagseq.getXTag(), 1)) 
			    ok = TRUE;
			else
			{
			    ok = FALSE;
			    cerr << "WLDB ERROR: unsupported attribute: " 
				 << tag << " " << tag.getTagName() << endl;
			    cerr << "            in worklist database file: " << fname << endl;
			    break;
			}
		    }
		}
	    }
	}
    }
    return ok;
}

static BOOL
isaWorklistFileName(const char* fname)
{
    if (fname == NULL)
	return FALSE;
    char* s = strrchr(fname, '.');
    if (s == NULL)
	return FALSE;
    if (strcmp(s, ".wl") == 0)
	return TRUE;
    return FALSE;
}

WDBStatus WorklistDB::startFindRequest(DcmDataset& findRequestIdentifiers)
{
    WDBStatus status = WDB_Success;
    BOOL match = FALSE;
    BOOL *seqmatch = NULL;

    delete offendingElements;
    delete errorElements;
    delete errorComment;
    offendingElements = new DcmAttributeTag(DCM_OffendingElement, 0);
    errorElements = new DcmAttributeTag(DCM_OffendingElement, 0);
    errorComment = new DcmLongString(DCM_ErrorComment, 0);

    this->clearDataset(identifiers);
    delete identifiers;
    identifiers = new DcmDataset(findRequestIdentifiers);
    identifiers->removeGroupLengthElements();

    this->clearObjectList();

    foundUnsupportedOptionalKey = FALSE;
       
    if (!this->checkIdentifiers(identifiers)) {
	/* bad identifiers */
	return WDB_Failed_IdentifierDoesNotMatchSOPClass;
    }
   
    const char * i1dname = dirpath;
    
    if (i1dname == NULL)
	return WDB_Refused_OutOfResorces;
    
    this -> set_readlock(i1dname);
    dirp = opendir(i1dname);
    if (dirp != NULL)
    {
	struct dirent *dp;
	for (dp = readdir(dirp); dp!=NULL; dp=readdir(dirp))
	{
	    if (isaWorklistFileName(dp->d_name))
	    {
		char file1[1024];
		strcpy(file1, i1dname);
		strcat(file1, "/");
		strcat(file1,dp -> d_name);
		cout << file1 << endl;
		
		DcmFileStream i1Stream(file1, DCM_ReadMode);
		
		if (i1Stream.Fail())
		{
		    cerr << "WLDB: cannot open worklist file: " << file1 << endl;
		    status =  WDB_Refused_OutOfResorces;
		}
		
		DcmFileFormat *fileform = new DcmFileFormat();
		fileform->transferInit();
		fileform->read(i1Stream, EXS_Unknown, EGL_withoutGL);
		fileform->transferEnd();
		if (fileform->error() != EC_Normal) {
		    fprintf(stderr, "WLDB: error: %s: reading worklist file: %s\n", 
			    dcmErrorConditionToString(fileform->error()), file1);
		    errorComment->put("Error in SCP Worklist Database");
		    status = WDB_Failed_UnableToProcess;
		    delete fileform;
		    break;
		}


		DcmDataset *dsetfile = fileform->getDataset();
		if (dsetfile == NULL) {
		    fprintf(stderr, "WLDB: error: empty worklist file: %s\n", file1);
		    errorComment->put("Error in SCP Worklist Database");
		    status = WDB_Failed_UnableToProcess;
		    delete fileform;
		    break;
		}

		dsetfile->removeGroupLengthElements();
		if (checkWklist(dsetfile, file1) != TRUE) {
		    delete fileform;
		    errorComment->put("Error in SCP Worklist Database");
		    status = WDB_Failed_UnableToProcess;
		    break;
		}
		unsigned long cdset = identifiers->card();
		
		for (unsigned long i = 0; i < cdset ; ++i)
		{
		    char *val1;
		    char *val2;
		    DcmElement *element = identifiers->getElement(i);
		    Uint32 laenge = element->getLength();
		    DcmTag tag(element->getTag());
		    
		    if (element->ident() != EVR_SQ)
		    {
			if (element->ident() != EVR_UL)
			{
			    if ((tag == DCM_PatientName)||(tag == DCM_PatientID))
			    {
				if (laenge == 0)
				    match = TRUE;
				else
				{
				    val2 = getvalue(element);
				    val1 = getvalue(dsetfile, tag);

				    if (matchPattern(val1, val2)) {
					match = TRUE;
				    } else {
					match = FALSE;
					break;
				    }
				}
			    }
			    else
			    {
				if (laenge == 0)
				{
				    match = TRUE;
				}
			    }
			}
			else 
			{
			    if (laenge == 0)
			    {
				match = TRUE;
			    }
			}
		    }
		    else
		    {
			if (element->ident() == EVR_SQ)
			{
			    if (laenge == 0)
				match = TRUE;
			    else
			    { 
				DcmSequenceOfItems *sequits = (DcmSequenceOfItems*)element;
				DcmItem *item = sequits -> getItem(0);
				Uint32 laengeofit = item->getLength();
				if (laengeofit == 0)
				    match = TRUE;
				else
				{
				    unsigned long cit = item -> card();
				    DcmStack stk1;
				    dsetfile->search(tag.getXTag(), stk1, ESM_fromHere, FALSE);
				    DcmObject *obj1 = stk1.top();
				    DcmSequenceOfItems *sequits1 = (DcmSequenceOfItems*)obj1;
				    unsigned long csequ1 = 0;
				    if (sequits1 != NULL)
					csequ1 = sequits1->card();
				    
				    seqmatch = new BOOL[csequ1];
				    for (unsigned long j = 0; j < csequ1; ++j)
				    {		     
					DcmItem *item1 = sequits1 -> getItem(j);
					DcmDataset *dset1 = (DcmDataset*)item1;
					
					for (unsigned long k = 0; k < cit; ++k)
					{ 
					    DcmElement *elementseq = item -> getElement(k);
					    if (elementseq->ident()!=EVR_UL)
					    {
						Uint32 laenge1 = elementseq -> getLength();
						DcmTag tagseq(elementseq->getTag());
						
						if ((tagseq == DCM_Modality) || 
						    (tagseq == DCM_ScheduledPerformingPhysiciansName) ||
						    (tagseq == DCM_ScheduledProcedureStepLocation) ||
						    (tagseq == DCM_CommentsOnTheScheduledProcedureStep)) 	  
						{
						    if (laenge1 == 0)
							seqmatch[j] = TRUE;
						    else
						    {
							val2 = getvalue(elementseq);
							val1 = getvalue(dset1, tagseq);

							if (matchPattern(val1, val2)) {
							    seqmatch[j] = TRUE;
							} else {
							    seqmatch[j] = FALSE;
							    break;
							}
						    }
						}
						else if ((tagseq == DCM_ScheduledStationAETitle) ||
							 (tagseq ==DCM_ScheduledStationName))
						{
						    if (laenge1 == 0)
							seqmatch[j] = TRUE;
						    else
						    {
							seqmatch[j] = FALSE; /* initial assumption */
							val2 = getvalue(elementseq);
							val1 = getvalue(dset1, tagseq);
							if (matchPattern(val1, val2)) {
							    /* the whole thing matches */
							    seqmatch[j] = TRUE;
							} else {
							    /* match against any multiple values */
							    unsigned long vm = getVMFromString(val1);
							    for (unsigned long v = 0; v < vm; v++)
							    {
								char * val3;
								val3 = getFirstValueFromString(val1);

								if (matchPattern(val3, val2)) {
								    seqmatch[j] = TRUE;
								} else {
								    seqmatch[j] = FALSE;
								}
								delete val3;
								if (seqmatch[j]) {
								    /* has matched one of the multiple 
								    ** values. break out of the vm loop.
								    */
								    break;
								}
							    }
							}
							if (!seqmatch[j]) {
							    /* has matched nothing */
							    break;
							}
						    }
						}
						else if (tagseq == DCM_ScheduledProcedureStepStartTime)
						{
						    if (laenge1 == 0)
							seqmatch[j] = TRUE;
						    else
						    {
							val2 = getvalue(elementseq);
							val1 = getvalue(dset1, tagseq);

							seqmatch[j] = matchTime(val1, val2);
							if (!seqmatch[j]) {
							    break;
							}
						    }
						}
						else if (tagseq == DCM_ScheduledProcedureStepStartDate)
						{ 
						    if (laenge1 == 0)
							seqmatch[j] = TRUE;
						    else
						    {
							val2 = getvalue(elementseq);
							val1 = getvalue(dset1, tagseq);

							seqmatch[j] = matchDate(val1, val2);
							if (!seqmatch[j]) {
							    break;
							}
						    }
						}
					    }
					}
				    }
				    for (int z = csequ1-1; z >= 0; z--)
				    {	
					if (seqmatch[z] == TRUE)
					{
					    match = TRUE;
					}
					else 
					{
					    DcmItem *item = sequits1 -> remove(z);
					    delete item;
					}
				    }
				    delete seqmatch;
				    if (sequits1 && (sequits1->card() == 0)) {
					match = FALSE;
					/* give up, nothing has matched */
					break;
				    }
				}
			    }
			}
		    }
		}
		if (match)
		{
		    objlist->append(fileform);
		    if (foundUnsupportedOptionalKey)
			status = WDB_Pending_Warning;
		    else
			status = WDB_Pending;
		} 
		else 
		    delete fileform;
	    }
	}
	closedir(dirp);
    }
    else
	status = WDB_Refused_OutOfResorces;

    this -> release_readlock();

    objlist->seek(ELP_first);
    return status;
}
 

    
    

// ********************************


WDBStatus WorklistDB::nextFindResponse(DcmDataset& findResponseIdentifiers)
{
    if (objlist->card() == 0)
	return WDB_Success;

    WDBStatus status = WDB_Failed_UnableToProcess;
    DcmElement *newElement = NULL;
    DcmItem *newItem = NULL;
    DcmDataset * dsetfile2 = new DcmDataset();
    DcmElement *el = (DcmElement*)objlist->seek_to(0);
    DcmFileFormat *file = (DcmFileFormat*)el;
    DcmDataset *set = file->getDataset();
    unsigned long int cids = identifiers->card();
        
    for (unsigned long int j = 0; j < cids; j++)
    {
	DcmElement *element = identifiers->getElement(j);
	DcmTag tag(element->getTag());
	if (element->ident() != EVR_SQ)
	{
	    if (isaSupportedReturnKey(tag.getXTag(), 0))
	    {
		newDicomElement(newElement, tag);
		char * val1 = getvalue(set, tag);
		newElement -> put(val1);
		dsetfile2 -> insert(newElement);
	    } else {
		/* insert an empty attribute */
		newDicomElement(newElement, tag);
		dsetfile2 -> insert(newElement);
	    }
	}
	else
	{
	    DcmSequenceOfItems *sequits = (DcmSequenceOfItems*)element;
	    newDicomElement(newElement, tag, 0);
	    DcmSequenceOfItems *newsequits = (DcmSequenceOfItems *)newElement;
	    dsetfile2 -> insert(newsequits);
	    DcmStack stk1;
	    set->search(tag.getXTag(), stk1, ESM_fromHere, FALSE);
	    DcmObject *obj1 = stk1.top();
	    DcmSequenceOfItems *sequits1 = (DcmSequenceOfItems*)obj1;
	    unsigned long csequ1 = 0;
	    if (sequits1)
		csequ1 = sequits1->card();
	    for (unsigned long int z = 0; z < csequ1; z++)
	    {
		DcmItem *item1 = sequits1->getItem(z);
		newItem = new DcmItem();
		newsequits -> insert(newItem);
		
		BOOL emptyIdentifiersItem = FALSE;
		if (sequits->card() == 0) {
		    emptyIdentifiersItem = TRUE;
		} else {
		    DcmItem *item = sequits->getItem(0);
		    if ((item == NULL) || (item->card() == 0)) {
			emptyIdentifiersItem = TRUE;
		    }
		}
		if (emptyIdentifiersItem)
		{
		    /* copy everying into the response */
		    unsigned long cit1 = item1->card();
		    for (unsigned long int w = 0; w < cit1; w++)
		    {
			DcmElement *elementseq1 = item1 -> getElement(w);
			DcmTag tagseq1(elementseq1->getTag());
			    
			if (isaSupportedReturnKey(tagseq1.getXTag(), 1))
			{
			    newDicomElement(newElement, tagseq1);
			    char *val1 = getvalue(elementseq1);
			    newElement -> put(val1);
			    newItem -> insert(newElement);
			} else {
			    /* insert an empty attribute */
			    newDicomElement(newElement, tagseq1);
			    newItem -> insert(newElement);
			}
		    }
		}
		else	    
		{
		    DcmItem *item = sequits->getItem(0);
		    unsigned long cit = item->card();

		    for (unsigned long k = 0; k < cit; ++k)
		    { 
			DcmElement *elementseq = item -> getElement(k);
			DcmTag tagseq(elementseq->getTag());
			
			if (isaSupportedReturnKey(tagseq.getXTag(), 1))
			{
			    newDicomElement(newElement, tagseq);
			    char *val1 = getvalue(item1, tagseq);
			    newElement -> put(val1);
			    newItem -> insert(newElement);
			} else {
			    /* insert an empty attribute */
			    newDicomElement(newElement, tagseq);
			    newItem -> insert(newElement);
			}
		    }
		}
	    }
	}
    }
    findResponseIdentifiers = *dsetfile2;
    if (objlist->card() > 0)
    {
	if (foundUnsupportedOptionalKey)
	    status = WDB_Pending_Warning;
	else
	    status = WDB_Pending;
    
	objlist->remove();
    }
    delete file;
    return status;
}




WDBStatus WorklistDB::cancelFindRequest()
{
   this -> clearObjectList();
  
   return WDB_Cancel;
}

const DcmAttributeTag* WorklistDB::getOffendingElements()
{
    if (offendingElements->getLength() == 0)
	return NULL;
    else
	return offendingElements;
}

const DcmLongString* WorklistDB::getErrorComments()
{
    if (errorComment->getLength() == 0)
	return NULL;
    else
	return errorComment;
}
       
	
/*
** CVS Log
** wrklstdb.cc,v
** Revision 1.8  1996/05/30 17:55:22  hewett
** Corrected processing of sequences in query identifiers.  Any and
** every sequence was being treated (and matched) as if it were a
** ScheduledProcedureStepSequence.
**
** Revision 1.7  1996/05/02 14:48:18  hewett
** Unsupported request identifiers as now returned in responses
** with no value.  If unsupported request identifiers are detected
** and a match is found then a status of Pending_Warning is generated
** instead of Pending.
**
** Revision 1.6  1996/05/02 13:42:10  hewett
** Revised error handling.  More accurate error messages when db is bad.
**
** Revision 1.5  1996/05/01 17:34:31  hewett
** Corrected some matching errors.  Removed some memory leaks.
**
** Revision 1.4  1996/04/30 13:57:44  savonge
** - more errors corrected
** - added checking of wklist database
**
** Revision 1.3  1996/04/29 19:15:08  hewett
** Added header comment and conditional inclusion of various .h files.
**
**
*/
